#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""
Created on Wed Oct 26 10:28:53 2016

@author: albert
"""
import logging
import string
import sys
import re
try:
    from synapseqrcodes_version import __version__
    assert __version__
except ImportError:
    __version__ = "UNKNOWN"
try:
    from synapseqrcodes_version import __revision__
    assert __revision__
except ImportError:
    __revision__ = "UNKNOWN"

SYNAPSE_LIGHTING_MODELS = [
    'DIM10-087-00',
    'DIM10-087-00-F',
    'DIM10-087-04',
    'DIM10-087-05',
    'DIM10-087-06',
    'DIM10-100-00',
    'DIM10-250-11',
    'DIM10-281-21',
    'DIM10-283-20',
    'AIM-121',
    'TL5-B1',
    'TL7-B1',
    'LP150-00',
    'OCF01-10T',
    'OCF01-1RT',
    'LP250V',
    'WSW-02',
    'WSW-08',
    'OCF00-1RN',
    ]

SYNAPSE_KEYS = [
    'qrcode format',
    'sales level part number',
    'date code',
    'ethernet mac address',
    'ethernet mac address last 6',
    'snap node address',
    'snap mac address',
    'wifi mac address',
    'model',
    'modem meid',
    'default password for user access',
    'default password for wifi access',
    'sim iccid',
    'batch size',
    'sequential code',
    'production order number',
    ]


def verify_output(output_dict):
    for key in output_dict:
        if not key in SYNAPSE_KEYS:
            raise ValueError('not a Synapse key: %s' % key)
    return output_dict


def verify_date_code(date_code):
    if len(date_code) != 4:
        raise ValueError('date code length is not 4')
    if not all(ch in string.digits for ch in date_code):
        raise ValueError('date code contains not only digits')


def verify_snap_node_address(snap_node_address):
    if len(snap_node_address) != 6:
        raise ValueError('snap node address length is not 6')
    if not all(ch in string.hexdigits for ch in snap_node_address):
        raise ValueError('snap node address contains not only hex digits')


def verify_ethernet_mac_address(ethernet_mac_address):
    if len(ethernet_mac_address) != 12:
        raise ValueError('ethernet node address length is not 12')
    if not all(ch in string.hexdigits for ch in ethernet_mac_address):
        raise ValueError('ethernet node address contains not hex digits')


def verify_ethernet_mac_address_last_6_digits(ethernet_mac_address):
    if len(ethernet_mac_address) != 6:
        raise ValueError('ethernet node address length is not 6')
    if not all(ch in string.hexdigits for ch in ethernet_mac_address):
        raise ValueError('ethernet node address contains not hex digits')


def verify_wifi_mac_address(wifi_mac_address):
    if len(wifi_mac_address) != 12:
        raise ValueError('wifi node address length is not 12')
    if not all(ch in string.hexdigits for ch in wifi_mac_address):
        raise ValueError('wifi node address contains not only hex digits')


def verify_snap_mac_address(snap_mac_address):
    if len(snap_mac_address) != 16:
        raise ValueError('snap mac address length is not 16')
    if not all(ch in string.hexdigits for ch in snap_mac_address):
        raise ValueError('snap mac address contains not only hex digits')


def verify_modem_meid(modem_meid):
    if len(modem_meid) != 14:
        raise ValueError('modem meid length is not 14')
    if not all(ch in string.hexdigits for ch in modem_meid):
        raise ValueError('modem meid contains not only hex digits')


def verify_sim_iccid(sim_iccid):
    if len(sim_iccid) != 20:
        raise ValueError('sim iccid length is not 14')
    if not all(ch in string.digits for ch in sim_iccid):
        raise ValueError('sim iccid contains not only digits')


def verify_lighting_model(model):
    if not model in SYNAPSE_LIGHTING_MODELS:
        raise ValueError('Unrecognized model')


def _synapse_qrcode_1(scan):
    """
    #01:AAAAAA:BBBB:CCCCCCCCCCCC

    THE #01 IS RESERVED FOR LIGHTING PRODUCTS ONLY
    Note that this is one only of only two formats where there is no colon at
    the end of the data
    Field A = SNAP MAC ADDRESS (6 characters)
    Field B = SYNAPSE MANUFACTURING DATE CODE (4 characters,
              first 2 characters are year, second 2 characters are week:
              Example, 0451 means week 51 of year 2004)
    Field C = This is a variable length alphanumeric field that represents the
              model number
    Actual product example…
    #01:072982:1602:DIM10-250-11
    Some products that use this are:
    DIM10-250-11
    Many other lighting products
    """
    if not scan.startswith('#01:'):
        raise ValueError('not starting with #01:')
    if scan.endswith(':'):
        raise ValueError('colon at the end of the data')
    scan_parts = scan.split(':')
    if len(scan_parts) == 4:
        snap_node_address = scan_parts[1]
        verify_snap_node_address(snap_node_address)
        date_code = scan_parts[2]
        verify_date_code(date_code)
        model = scan_parts[3]
        verify_lighting_model(model)
        if not model in SYNAPSE_LIGHTING_MODELS:
            raise ValueError('Unrecognized model')
        return verify_output(
            {
                'qrcode format': scan_parts[0],
                'snap node address': snap_node_address,
                'date code': date_code,
                'model': model,
                })
    else:
        raise ValueError('wrong number of colons')


def _synapse_qrcode_2(scan):
    """
    #02:AAAAAAAAAAAAAAAAAAAA:BBBB:CCCCCCCCCCCC:\
        DDDDDDDDDDDDDDDD:EEEEEEEEEEEE:FFFFFFFFFFFFFF

    The #02 Format is for E20 products
    Note that this is one only of only two formats where there is no colon at
    the end of the data
    Field A = SALES LEVEL PART NUMBER (up to 20 characters)
    Field B = SYNAPSE MANUFACTURING DATE CODE (4 characters,
              first 2 characters are year, second 2 characters are week:
              Example, 0451 means week 51 of year 2004)
    Field C = ETHERNET MAC ADDRESS (12 characters)
    Field D = SNAP MAC ADDRESS (16 characters)
    Field E = WIFI MAC ADDRESS (12 characters)
    Field F = MODEM MEID (up to 14 characters); ‘0’ if not used
    Actual product example…
    #02:SC020‑020:1619:001C2C618B80:001C2C1B2608527E:00078006DE35:A1000042F1866B
    NOTE: due to a mistake initially, the E20’s without cell modem are
    formatted without field F at all.  QR code readers will have to be aware of
    the missing field.  Below is an example.
    All E20’s without cell modems will continue with this mistake for
    the sake of being consistent in format.
    #02:SC020-010:1618:001C2C62BB27:001C2C1B26085385:00078006E3DD
    Some products that use this are:
    E20 and variants
    """
    if not scan.startswith('#02:'):
        raise ValueError('not starting with #02:')
    if scan.endswith(':'):
        raise ValueError('colon at the end of the data')
    scan_parts = scan.split(':')
    if len(scan_parts) == 7:
        sales_level_part_number = scan_parts[1]
        # TBD is there a list of sales level part numbers of E20 variants?
        if len(sales_level_part_number) > 20:
            raise ValueError('sales level part number is too long')
        date_code = scan_parts[2]
        verify_date_code(date_code)
        ethernet_mac_address = scan_parts[3]
        verify_ethernet_mac_address(ethernet_mac_address)
        snap_mac_address = scan_parts[4]
        verify_snap_mac_address(snap_mac_address)
        wifi_mac_address = scan_parts[5]
        verify_wifi_mac_address(wifi_mac_address)
        modem_meid = scan_parts[6]
        verify_modem_meid(modem_meid)
        return verify_output(
            {
                'qrcode format': scan_parts[0],
                'sales level part number': sales_level_part_number,
                'date code': date_code,
                'ethernet mac address': ethernet_mac_address,
                'snap mac address': snap_mac_address,
                'wifi mac address': wifi_mac_address,
                'modem meid': modem_meid,
                })
    elif len(scan_parts) == 6:
        sales_level_part_number = scan_parts[1]
        # TBD is there a list of sales level part numbers of E20 variants?
        if len(sales_level_part_number) > 20:
            raise ValueError('sales level part number is too long')
        date_code = scan_parts[2]
        verify_date_code(date_code)
        ethernet_mac_address = scan_parts[3]
        verify_ethernet_mac_address(ethernet_mac_address)
        snap_mac_address = scan_parts[4]
        verify_snap_mac_address(snap_mac_address)
        wifi_mac_address = scan_parts[5]
        verify_wifi_mac_address(wifi_mac_address)
        return verify_output(
            {
                'qrcode format': scan_parts[0],
                'sales level part number': sales_level_part_number,
                'date code': date_code,
                'ethernet mac address': ethernet_mac_address,
                'snap mac address': snap_mac_address,
                'wifi mac address': wifi_mac_address,
                'modem meid': '0',
                })
    else:
        raise ValueError('wrong number of colons')


def _synapse_qrcode_3(scan):
    """
    #03:AAAAAAAAAAAAAAAAAAAA:BBBB:CCCCCCCCCCCC:\
        DDDDDDDDDDDDDDDD:0:FFFFFFFFFFFFFFFFFFFFFFFFFF:
    The #03 Format is for SNAP Link products
    Don’t forget the colon on the end of the data
    Field A = SALES LEVEL PART NUMBER (up to 20 characters)
    Field B = SYNAPSE MANUFACTURING DATE CODE (4 characters,
              first 2 characters are year, second 2 characters are week:
              Example, 0451 means week 51 of year 2004)
    Field C = ETHERNET MAC ADDRESS (12 characters); ‘0’ if not used
    Field D = SNAP MAC ADDRESS (16 characters)
    Field E = ‘0’
    Field F = MODEL (up to 26 characters)
    Example…
    #03:SN232-003:1610:0:001C2C1B26605BA5:0:SNAPLINK:
    Some products that use this are:
    SL232-003 (SNAP Link 232, FCC only)
    SL485-003 (SNAP Link 485, FCC only)
    SL232-004 (SNAP Link 232, CE)
    SL485-004 (SNAP Link 485, CE)
    """
    if not scan.startswith('#03:'):
        raise ValueError('not starting with #03:')
    if not scan.endswith(':'):
        raise ValueError('no colon at the end of the data')
    scan_parts = scan.split(':')
    if len(scan_parts) == 8:
        sales_level_part_number = scan_parts[1]
        # TBD is there a list of sales level part numbers
        # for SNAP Link products?
        if len(sales_level_part_number) > 20:
            raise ValueError('sales level part number is longer than 20')
        date_code = scan_parts[2]
        verify_date_code(date_code)
        ethernet_mac_address = scan_parts[3]
        if ethernet_mac_address != '0':
            verify_ethernet_mac_address(ethernet_mac_address)
        snap_mac_address = scan_parts[4]
        verify_snap_mac_address(snap_mac_address)
        field_e = scan_parts[5]
        if field_e != '0':
            raise ValueError('Field E is not zero')
        model = scan_parts[6]
        # TBD is there a list of models
        # for SNAP Link products?
        if len(model) > 26:
            raise ValueError('model is longer than 26')
        return verify_output(
            {
                'qrcode format': scan_parts[0],
                'sales level part number': sales_level_part_number,
                'date code': date_code,
                'ethernet mac address': ethernet_mac_address,
                'snap mac address': snap_mac_address,
                'model': model,
                })
    else:
        raise ValueError('wrong number of colons')


def _synapse_qrcode_4(scan):
    """
    #04:AAAAAAAAAAAAAAAAAAAA:BBBB:CCCCCCCCCCCC:\
        DDDDDDDDDDDDDDDD:0:FFFFFFFFFFFFFFFFFFFFFFFFFF:
    The #04 Format is for E12 products
    Don’t forget the colon on the end of the data
    Field A = SALES LEVEL PART NUMBER (up to 20 characters)
    Field B = SYNAPSE MANUFACTURING DATE CODE (4 characters,
              first 2 characters are year, second 2 characters are week:
              Example, 0451 means week 51 of year 2004)
    Field C = ETHERNET MAC ADDRESS (12 characters); ‘0’ if not used
    Field D = SNAP MAC ADDRESS (16 characters)
    Field E = ‘0’
    Field F = MODEL (up to 26 characters)
    Example…
    #03:SC012-010:1610: 00078006DE35:001C2C1B26605BA5:0:E12:
    Some products that use this are:
    E12
    """
    if not scan.startswith('#04:'):
        raise ValueError('not starting with #04:')
    if not scan.endswith(':'):
        raise ValueError('no colon at the end of the data')
    scan_parts = scan.split(':')
    if len(scan_parts) == 8:
        sales_level_part_number = scan_parts[1]
        # TBD is there a list of sales level part numbers
        # for E12 products?
        if len(sales_level_part_number) > 20:
            raise ValueError('sales level part number is longer than 20')
        date_code = scan_parts[2]
        verify_date_code(date_code)
        ethernet_mac_address = scan_parts[3]
        verify_ethernet_mac_address(ethernet_mac_address)
        snap_mac_address = scan_parts[4]
        verify_snap_mac_address(snap_mac_address)
        field_e = scan_parts[5]
        if field_e != '0':
            raise ValueError('Field E is not zero')
        model = scan_parts[6]
        # TBD is there a list of models
        # for E12 products?
        if len(model) > 26:
            raise ValueError('model is longer than 26')
        return verify_output(
            {
                'qrcode format': scan_parts[0],
                'sales level part number': sales_level_part_number,
                'date code': date_code,
                'ethernet mac address': ethernet_mac_address,
                'snap mac address': snap_mac_address,
                'model': model,
                })
    else:
        raise ValueError('wrong number of colons')


def _synapse_qrcode_5(scan):
    """
    #05:AAAAAAAAAAAAAAAAAAAA:BBBBBBBBBBBBBBBBBBBB:
    The #05 Format is for SS420, SS450, and other SimplySNAP products
    Don’t forget the colon on the end of the data
    Field A = Default Password for User Access (12 - 20 characters)
    Field B = Default Password for WIFI Access (12 - 20 characters)
    Actual product example…
    #05:Sassy$Suzzyq78:Silent&Tim45:
    """
    if not scan.startswith('#05:'):
        raise ValueError('not starting with #05:')
    if not scan.endswith(':'):
        raise ValueError('no colon at the end of the data')
    scan_parts = scan.split(':')
    if len(scan_parts) == 4:
        return verify_output(
            {
                'qrcode format': scan_parts[0],
                'default password for user access': scan_parts[1],
                'default password for wifi access': scan_parts[2],
                })
    else:
        raise ValueError('wrong number of colons')


def _synapse_qrcode_6(scan):
    """
    #06:SC012-030:1650:63B704:
    The #06 Format is for E12 products w/o housing
    Don’t forget the colon on the end of the data
    Field A = SALES LEVEL PART NUMBER (up to 9 characters)
    Field B = SYNAPSE MANUFACTURING DATE CODE (4 characters,
              first 2 characters are year, second to characters are week:
              Example, 0451 means week 51 of year 2004)
    Field C = LAST 6 CHARACTERS OF THE ETHERNET MAC ADDRESS (6 characters)
    Example…
    #06:SC012-030:1650:63B704:
    Some products this is used on:
    SC012-030 (E12 SNAPconnect Gateway – w/o housing – North America)
    SC012-040 (E12 SNAPconnect Gateway – w/o housing – Europe)
    """
    if not scan.startswith('#06:'):
        raise ValueError('not starting with #06:')
    if not scan.endswith(':'):
        raise ValueError('no colon at the end of the data')
    scan_parts = scan.split(':')
    if len(scan_parts) == 5:
        return verify_output(
            {
                'qrcode format': scan_parts[0],
                'sales level part number': scan_parts[1],
                'date code': scan_parts[2],
                'ethernet mac address last 6': scan_parts[3],
                })
    else:
        raise ValueError('wrong number of colons')


def _synapse_qrcode_7(scan):
    """
    #07:SN087-001:1650:63B704:
    The #07 Format is for SN087 products
    Don’t forget the colon on the end of the data
    Field A = SALES LEVEL PART NUMBER (up to 9 characters)
    Field B = SYNAPSE MANUFACTURING DATE CODE (4 characters,
              first 2 characters are year, second to characters are week:
              Example, 0451 means week 51 of year 2004)
    Field C = LAST 6 CHARACTERS OF THE SNAP MAC ADDRESS (6 characters)
    Example…
    #07:SN087-001:1650:63B704:
    Some products this is used on:
    SN087-001
    """
    if not scan.startswith('#07:'):
        raise ValueError('not starting with #07:')
    if not scan.endswith(':'):
        raise ValueError('no colon at the end of the data')
    scan_parts = scan.split(':')
    if len(scan_parts) == 5:
        return verify_output(
            {
                'qrcode format': scan_parts[0],
                'sales level part number': scan_parts[1],
                'date code': scan_parts[2],
                'snap node address': scan_parts[3],
                })
    else:
        raise ValueError('wrong number of colons')


def _synapse_qrcode_8(scan):
    """
    #08:AAAAAAAAAAAAAAAAAAAA:BBBB:CCCCCC:DDDDDD:EEEEEEEEEEEE:\
        FFFFFFFFFFFFFFFF:GGGGGGGGGGGGGGGGGGGG:
    The #08 Format is for E20 products with a 4G LTE modem
    Note: the Ethernet and SNAP MAC addresses were shortened
    to be able to keep the IMEI and ICCID in the QR code
    Don’t forget the colon on the end of the data
    Field A = SALES LEVEL PART NUMBER (up to 20 characters)
    Field B = SYNAPSE MANUFACTURING DATE CODE (4 characters,
              first 2 characters are year,
              second 2 characters are week
              Example: 0451 means week 51 of year 2004)
    Field C = ETHERNET MAC ADDRESS (6 characters)
    Field D = SNAP MAC ADDRESS (6 characters)
    Field E = WIFI MAC ADDRESS (12 characters)
    Field F = MODEM IMEI (up to 16 characters)
    Field G = SIM ICCID (up to 20 characters)
    Here is an example…
    #08:SC020-020:1619:618B80:08527E:00078006DE35:\
        356278079904498:89148000001081675984:
    Some products that use this are:
        E20
        SS450-002
    """
    if not scan.startswith('#08:'):
        raise ValueError('not starting with #08:')
    if not scan.endswith(':'):
        raise ValueError('no colon at the end of the data')
    scan_parts = scan.split(':')
    if len(scan_parts) == 9:
        sales_level_part_number = scan_parts[1]
        # TBD is there a list of sales level part numbers of E20 variants?
        if len(sales_level_part_number) > 20:
            raise ValueError('sales level part number is too long')
        date_code = scan_parts[2]
        verify_date_code(date_code)
        ethernet_mac_address_last_6 = scan_parts[3]
        verify_ethernet_mac_address_last_6_digits(ethernet_mac_address_last_6)
        snap_node_address = scan_parts[4]
        verify_snap_node_address(snap_node_address)
        wifi_mac_address = scan_parts[5]
        verify_wifi_mac_address(wifi_mac_address)
        modem_meid = scan_parts[6]
        verify_modem_meid(modem_meid)
        sim_iccid = scan_parts[7]
        verify_sim_iccid(sim_iccid)
        return verify_output(
            {
                'qrcode format': scan_parts[0],
                'sales level part number': sales_level_part_number,
                'date code': date_code,
                'ethernet mac address last 6': ethernet_mac_address_last_6,
                'snap node address': snap_node_address,
                'wifi mac address': wifi_mac_address,
                'modem meid': modem_meid,
                'sim iccid': sim_iccid,
                }
            )
    else:
        raise ValueError('wrong number of colons')


def _synapse_qrcode_eaton_ss420_003e(scan):
    """full specification in the attached DOC-1000123"""
    if not re.match('\A..;.;10021361;SS420-003E', scan):
        raise ValueError('not an EATON SS420-003E QR code')
    scan_parts = scan.replace(';', ',').split(',')
    if len(scan_parts) != 14:
        raise ValueError('wrong number of parts: %d' % len(scan_parts))
    # 1361of2000
    seq_parts = scan_parts[9].replace('of', ',').split(',')
    if len(seq_parts) != 2:
        raise ValueError('wrong sequence')
    sequential_code, batch_size = seq_parts
    # TBD is there a list of sales level part numbers of E20 variants?
    production_order_number = scan_parts[5]
    date_code = scan_parts[8] + scan_parts[7]
    verify_date_code(date_code)
    ethernet_mac_address = scan_parts[10]
    verify_ethernet_mac_address(ethernet_mac_address)
    snap_node_address = scan_parts[11]
    verify_snap_node_address(snap_node_address)
    return verify_output(
        {
            'qrcode format': 'DOC-1000123',
            'sales level part number': 'SS420-003E',
            'production order number': production_order_number,
            'date code': date_code,
            'ethernet mac address': ethernet_mac_address,
            'snap node address': snap_node_address,
            'sequential code': sequential_code,
            'batch size': batch_size,
            'default password for user access': scan_parts[13],
            'default password for wifi access': scan_parts[12],
            }
        )


def synapse_qrcode(scan):
    if scan.startswith("#01:"):
        return _synapse_qrcode_1(scan)
    elif scan.startswith("#02:"):
        return _synapse_qrcode_2(scan)
    elif scan.startswith("#03:"):
        return _synapse_qrcode_3(scan)
    elif scan.startswith("#04:"):
        return _synapse_qrcode_4(scan)
    elif scan.startswith("#05:"):
        return _synapse_qrcode_5(scan)
    elif scan.startswith("#06:"):
        return _synapse_qrcode_6(scan)
    elif scan.startswith("#07:"):
        return _synapse_qrcode_7(scan)
    elif scan.startswith("#08:"):
        return _synapse_qrcode_8(scan)
    elif re.match('\A..;.;10021361;SS420-003E', scan):
        return _synapse_qrcode_eaton_ss420_003e(scan)
    else:
        raise ValueError('QRcode cannot be recognized')


def synapse_qrcode_loop():
    logging.basicConfig(level=logging.INFO)
    logging.info('synapseqrcodes version: %s', __version__)
    logging.info('synapseqrcodes revision: %s', __revision__)
    while True:
        try:
            scan = raw_input('\nScan Synapse QRcode (x to exit):')
            if scan.upper() == 'X':
                break
            qrcode = synapse_qrcode(scan)
            sys.stderr.write(repr(qrcode))
        except ValueError:
            logging.exception('ERROR')


if __name__ == "__main__":
    synapse_qrcode_loop()
